function saveOptions() {
  let oab = document.getElementById('oab').checked;
  let ill = document.getElementById('ill').checked;
  if(oab && ill){
    oab = false;
    document.getElementById('oab').checked = false;
  }
  chrome.storage.local.set({
    core: document.getElementById('core').checked,
    core1: document.getElementById('core1').checked,
    oab,
    ezproxy: document.getElementById('ezproxy').checked,
    ill,
    search: document.getElementById('selectsearch').value,
    opencitation: document.getElementById('opencitation').checked,
    nobadge: document.getElementById('nobadge').checked,
  }, () => {
    // Update status to let user know options were saved.
    const status = document.getElementById('status');
    status.innerHTML = chrome.i18n.getMessage('options_options_saved');
    clearStatus();
  });
}

function saveInstitute(id, name, url, illurl, domainurl) {
  document.getElementById('institute').value = `${name} (${id})`;
  let lastload = Math.floor(Date.now() / 1000);
  let ill = false;
  if(illurl.length > 0 && illurl.indexOf('http') === 0) {
    ill = true;
  }
  let ezproxy = false;
  if(url.length > 0 && url.indexOf('http') === 0){
    ezproxy = true;
  }
  chrome.storage.local.set({
    id,
    name,
    url,
    illurl,
    ezproxy,
    ill,
    domainurl,
    lastload
  }, () => {
    // Update status to let user know options were saved.
    restoreOptions();
    const status = document.getElementById('status');
    status.innerHTML = chrome.i18n.getMessage('options_options_proxy_saved', [id, name]);
    const proxyurl = document.getElementById('proxyurl');
    proxyurl.textContent = url;
    const illurlText = document.getElementById('illurl');
    illurlText.textContent = illurl;
    clearStatus();
    if (domainurl != '') {
      chrome.runtime.sendMessage({ mode: 'getProxyDomains', id, domainurl }, (response) => {
        console.log(response);
      });
    }

  });
}

function restoreOptions() {
  chrome.storage.local.get({
    core, core1, oab, ezproxy, ill, opencitation, nobadge,
  }, (items) => {
    for (const [key, value] of Object.entries(items)) {
      //console.log(`${key} - ${value}`);
      document.getElementById(key).checked = value;
    }
  });
}

function restoreDropDown() {
  chrome.storage.local.get(['search'], (items) => {
    for (const [key, value] of Object.entries(items)) {
      const select = document.getElementById('selectsearch');
      for (let i = 0; i < select.options.length; i++) {
        if (select.options[i].value == value) {
          select.options[i].selected = true;
        }
      }
    }
  });
}

function restoreInstitute() {
  chrome.storage.local.get(['id', 'name', 'url', 'illurl'], (items) => {
    for (const [key, value] of Object.entries(items)) {
      //console.log(`${key} - ${value}`);
      document.getElementById('institute').value = `${items.name} (${items.id})`;
      const proxyurl = document.getElementById('proxyurl');
      proxyurl.textContent = items.url;
      const illurl = document.getElementById('illurl');
      illurl.textContent = items.illurl;
    }
  });
}

function setup() {
  chrome.storage.local.get(['issetup'], (items) => {
    if (items.issetup != undefined && items.issetup) {
      restoreOptions();
      restoreInstitute();
      restoreDropDown();
      showHideUpdateInstitute();
    } else {
      doInitialSetup();
    }
  });
}

function doInitialSetup() {
  chrome.storage.local.set({
    core: true,
    core1: false,
    oab: true,
    opencitation: true,
    ezproxy: false,
    ill: false,
    issetup: true,
    search: 'google',
    consolelog: false,
    nobadge: false,
  }, () => {
    restoreOptions();
    restoreDropDown();
    showHideUpdateInstitute();
  });
}

function showHideUpdateInstitute() {
  chrome.storage.local.get(['id'], (items) => {
    if(items.id != undefined && items.id > 0){
      document.getElementById('instituteupdate').style.display = "inline-block";
    }
    else{
      document.getElementById('instituteupdate').style.display = "none";
    }
  });
}


async function doProxySearch() {
  const searchTerm = document.getElementById('institute').value;
  if (searchTerm.length < 5) {
    //console.log('too short');
    const response = { code: 203, data: [] };
    return response;
  }
  const url = 'https://www.oahelper.org/backend/institutes/api.php';
  const apiKey = 'l6sNUJ3Knh';
  const formData = new URLSearchParams();
  let searchtype = 'domain';
  if (document.getElementById('query').checked) {
    searchtype = document.getElementById('query').value;
  }
  formData.append(searchtype, searchTerm);
  const response = await fetch(url, {
    method: 'POST',
    mode: 'cors',
    cache: 'no-cache',
    credentials: 'same-origin',
    headers: {
	  'Content-Type': 'application/x-www-form-urlencoded',
	  'X-Token': apiKey,
    },
    redirect: 'follow',
    referrerPolicy: 'no-referrer',
    body: formData,
  });
  return response.json();
}

function proxySearch() {
  document.getElementById('institutelist').innerHTML = '';
  doProxySearch().then((response) => {
    if (response != undefined && (response.code >= 200 && response.code <= 299)) {
      if (response.code == 203) {
        const status = document.getElementById('status');
        status.innerHTML = chrome.i18n.getMessage('options_proxy_searchtooshort');
        // 'Please enter at least 5 characters';
        clearStatus();
      } else if (response.data.length == 0) {
        // we found nothing
        const status = document.getElementById('status');
        status.innerHTML = chrome.i18n.getMessage('options_no_institutes');
        clearStatus();
      } else if (response.data.length == 1) {
        // we found one
        saveInstitute(response.data[0].id, response.data[0].institution, response.data[0].proxyUrl.replace('{targetUrl}', ''), response.data[0].ill.replace('{doi}', ''), response.data[0].domainUrl);
        // don't need a message, as that will be provided by saveInstitute function
      } else {
        // we found multiple
        const status = document.getElementById('status');
        status.innerHTML = chrome.i18n.getMessage('options_multi_institutes');
        const html = [];
        response.data.forEach((data) => {
          html.push(`<tr class="institutetoadd" data-id="${data.id}" data-institute="${data.institution}" data-url="${data.proxyUrl.replace('{targetUrl}', '')}" data-ill="${data.ill.replace('{doi}','')}" data-domainurl="${data.domainurl}"><td>${data.id}</td><td>${data.institution}</td><td>${data.proxyUrl.replace('{targetUrl}', '')}</td><td>${data.ill.replace('{doi}', '')}</td><tr>`);
        });
        const rows = html.join('');
        document.getElementById('institutelist').innerHTML = `<table><thead><tr><th>ID</th><th>Institute</th><th>URL</th><th>ILL</th></tr></thead><tbody>${rows}</tbody></table>`;
        const items = document.getElementsByClassName('institutetoadd');
        for (let i = 0; i < items.length; i++) {
          items[i].addEventListener('click', itemToAddClicked);
        }
        clearStatus();
      }
    } else {
      const status = document.getElementById('status');
      status.innerHTML = chrome.i18n.getMessage('options_error_institutes');
      clearStatus();
    }
  });
}

async function doProxySearchById(searchTerm) {
  const url = 'https://www.oahelper.org/backend/institutes/api.php';
  const apiKey = 'l6sNUJ3Knh';
  const formData = new URLSearchParams();
  let searchtype = 'id';
  formData.append(searchtype, searchTerm);
  const response = await fetch(url, {
    method: 'POST',
    mode: 'cors',
    cache: 'no-cache',
    credentials: 'same-origin',
    headers: {
	  'Content-Type': 'application/x-www-form-urlencoded',
	  'X-Token': apiKey,
    },
    redirect: 'follow',
    referrerPolicy: 'no-referrer',
    body: formData,
  });
  return response.json();
}


function reloadSettings() {
  chrome.storage.local.get(['id'], (items) => {
    doProxySearchById(items.id).then((response) => {
      if (response != undefined && (response.code >= 200 && response.code <= 299)) {
        if (response.data.length == 0) {
          // we found nothing
          const status = document.getElementById('status');
          status.innerHTML = chrome.i18n.getMessage('options_no_institutes');
          clearStatus();
        } else{
          // we found one, as we search by id there can only be one result
          saveInstitute(response.data[0].id, response.data[0].institution, response.data[0].proxyUrl.replace('{targetUrl}', ''), response.data[0].ill.replace('{doi}', ''), response.data[0].domainUrl);
          // don't need a message, as that will be provided by saveInstitute function
        } 
      } else {
        const status = document.getElementById('status');
        status.innerHTML = chrome.i18n.getMessage('options_error_institutes');
        clearStatus();
      }
    });
  });
}

function itemToAddClicked() {
  saveInstitute(this.dataset.id, this.dataset.institute, this.dataset.url, this.dataset.ill, this.dataset.domainurl);
}

function clearStatus() {
  setTimeout(() => {
    document.getElementById('status').innerHTML = '';
  }, 2500);
}

function showsettings() {
  document.getElementById('settingssection').style.display = 'block';
  document.getElementById('showsettings').style.display = 'none';
  document.getElementById('hidesettings').style.display = 'inline-block';
}

function hidesettings() {
  document.getElementById('settingssection').style.display = 'none';
  document.getElementById('showsettings').style.display = 'inline-block';
  document.getElementById('hidesettings').style.display = 'none';
}

function viewtutorial() {
  chrome.tabs.create({ url: 'https://www.oahelper.org/tutorial-open-access-helper-for-chrome/' }, (tab) => {
    console.log('tutorial has been opened');
  });
}

function getsupport() {
  chrome.tabs.create({ url: 'https://www.oahelper.org/support/' }, (tab) => {
    console.log('support has been opened');
  });
}

function showexample() {
  chrome.tabs.create({ url: 'https://www.oahelper.org/example/' }, (tab) => {
    console.log('example has been opened');
  });
}


function localizeHtmlPage() {
  // Localize by replacing __MSG_***__ meta tags
  const objects = document.getElementsByTagName('html');
  for (let j = 0; j < objects.length; j++) {
    const obj = objects[j];

    const valStrH = obj.innerHTML.toString();
    const valNewH = valStrH.replace(/__MSG_(\w+)__/g, (match, v1) => (v1 ? chrome.i18n.getMessage(v1) : ''));

    if (valNewH != valStrH) {
      obj.innerHTML = valNewH;
    }
  }
}

localizeHtmlPage();


document.addEventListener('DOMContentLoaded', setup);
document.getElementById('save').addEventListener('click', saveOptions);
document.getElementById('proxysearch').addEventListener('click', proxySearch);
document.getElementById('instituteupdate').addEventListener('click', reloadSettings);
// document.getElementById('showsettings').addEventListener('click', showsettings);
document.getElementById('showexample').addEventListener('click', showexample);
document.getElementById('viewtutorial').addEventListener('click', viewtutorial);
document.getElementById('getsupport').addEventListener('click', getsupport);

