
function localizeHtmlPage() {
  // Localize by replacing __MSG_***__ meta tags
  const objects = document.getElementsByTagName('html');
  for (let j = 0; j < objects.length; j++) {
    const obj = objects[j];

    const valStrH = obj.innerHTML.toString();
    const valNewH = valStrH.replace(/__MSG_(\w+)__/g, (match, v1) => (v1 ? chrome.i18n.getMessage(v1) : ''));

    if (valNewH != valStrH) {
      obj.innerHTML = valNewH;
    }
  }
}

localizeHtmlPage();

chrome.windows.getAll({
  populate: true,
}, (windows) => {
  // For all windows
  for (let i = 0; i < windows.length; i++) {
    const currentWindow = windows[i];
    // For all tabs in that window
    for (let j = 0; j < currentWindow.tabs.length; j++) {
      const currentTab = currentWindow.tabs[j];
      // Only send message to http or https pages, not ftp://, chrome:// etc.
      if (currentTab.active && currentTab.url.match(/(http|https):\/\//gi)) {
        chrome.tabs.sendMessage(currentTab.id, { mode: 'popup' }, (response) => {
          const element = document.getElementById('popupanswer');

          let oabutton = '';
          let citationbutton = '';
          let proxybutton = '';
          let institute = '';
          let buttonCount = 0;
          let illButtonCount = 0;
          if (response != undefined && response.data != undefined && response.data.oaurl != '' && response.data.oastatus != '') {
            if (response.data.oaurl.indexOf('https://openaccessbutton.org/request?') === 0) {
              oabutton = chrome.i18n.getMessage('popup_oabutton_001', [response.data.oaurl, response.data.oastatus]);
              buttonCount += 1;
            } else if(response.data.isIll === 'ill') {
              oabutton = chrome.i18n.getMessage('popup_oabutton_003', [response.data.oaurl, response.data.oastatus]);
              buttonCount += 1;
              illButtonCount += 1;
            } else if(response.data.isIll === '✔'){
              oabutton = chrome.i18n.getMessage('popup_oabutton_006', [response.data.oaurl, response.data.oastatus]);
              buttonCount += 1;
            } 
            else {
              oabutton = chrome.i18n.getMessage('popup_oabutton_002', [response.data.oaurl, response.data.oastatus]);
              buttonCount += 1;
            }
          }
          if (response != undefined && response.data != undefined && response.data.citationcount > 0) {
            citationbutton = chrome.i18n.getMessage('popup_citationbutton_001', [response.data.citationurl, response.data.citationcount]);
            buttonCount += 1;
          }

          if (response != undefined && response.configuration != undefined && response.configuration.name != undefined && response.configuration.name != '') {
            institute = response.configuration.name;
          }

          if (response != undefined && response.configuration != undefined && response.configuration.ezproxy && response.configuration.url != '' && !onProxiedDomain(response.configuration.url, currentTab.url) && response.data.shouldProxy) {
            const newUrl = new URL(currentTab.url);
            let urltoproxy = newUrl.hostname;
            if (urltoproxy.length > 25) {
              urltoproxy = `${urltoproxy.substring(0, 25)}&hellip;`;
            }
            const proxyUrl = response.configuration.url + currentTab.url;
            proxybutton = chrome.i18n.getMessage('popup_proxybutton_001', [institute, proxyUrl, urltoproxy]);
            buttonCount += 1;
          }

          
          let addonButton = '';
          if(response.configuration.ill && response.data.doi != '' && response.data.doi != '-' && illButtonCount == 0){
            let illUrl = response.configuration.illurl + response.data.doi;
            let illLabel = chrome.i18n.getMessage('content_ill_002');
            
            if(buttonCount < 3){
              //we will simply add a new button
              addonButton = chrome.i18n.getMessage('popup_oabutton_005', [illUrl, illLabel]);
            }
            else{
              addonButton = chrome.i18n.getMessage('popup_oabutton_004', [illUrl, illLabel])
            }
          }

          let html = `${oabutton}${citationbutton}${proxybutton}${addonButton}`;

          if (html == '' && onProxiedDomain(response.configuration.url, currentTab.url)) {
            html = chrome.i18n.getMessage('popup_empty_responses_proxied');
          }
          else if(html == '') {
            html = chrome.i18n.getMessage('popup_empty_responses');
          }
          element.innerHTML = html;
          addButtonListener();
          addConfigurationListener();
          addSupportListener();
          addHideListener();
        });
      } else {
        const element = document.getElementById('popupanswer');
        html = chrome.i18n.getMessage('popup_non_http_source');
        element.innerHTML = html;
        addConfigurationListener();
        addSupportListener();
        addHideListener();
      }
    }
  }
});

function onProxiedDomain(proxyUrl, currentUrl){
  //we take apart the proxy url and remove the first part of it,
  //as it could be something like login.proxy.uni.edu
  //the finaly proxied url then only contains domain-tld.proxy.uni.edu
  if (proxyUrl == '') {
    return false;
  }
  const ezproxyUrl = new URL(proxyUrl);
  const urlParts = ezproxyUrl.hostname.split('.');
  urlParts.splice(0,1);
  const testUrl = urlParts.join('.');
  if(currentUrl.indexOf(testUrl) >= 0){
    return true;
  }
  return false;
}

function addButtonListener() {
  const links = document.getElementsByTagName('button');
  for (var i = 0; i < links.length; i++) {
    (function () {
      const ln = links[i];
      const location = ln.dataset.url;
      ln.onclick = function () {
        chrome.tabs.create({ active: true, url: location });
      };
    }());
  }
}

function addConfigurationListener() {
  const configLink = document.getElementById('configurationlink');
  configLink.onclick = function () {
    chrome.tabs.create({ active: true, url: chrome.runtime.getURL('html/options.html') });
  };

  const configIcon = document.getElementById('oah_settings_icon');
  configIcon.onclick = function () {
    chrome.tabs.create({ active: true, url: chrome.runtime.getURL('html/options.html') });
  };
}

function addSupportListener() {
  const configLink = document.getElementById('supportlink');
  configLink.onclick = function () {
    chrome.tabs.create({ active: true, url: 'https://www.oahelper.org/support/' });
  };
}

function addHideListener() {
  const configLink = document.getElementById('hidebadges');
  configLink.onclick = function () {
    document.getElementById('hidebadgespan').style.display = 'none';
    chrome.windows.getAll({ populate: true }, (windows) => {
      // For all windows
      for (let i = 0; i < windows.length; i++) {
        const currentWindow = windows[i];
        // For all tabs in that window
        for (let j = 0; j < currentWindow.tabs.length; j++) {
          const currentTab = currentWindow.tabs[j];
          if (currentTab.active && currentTab.url.match(/(http|https):\/\//gi)) {
            chrome.tabs.sendMessage(currentTab.id, { mode: 'hidebadges' }, () => {
              // console.log(response);
            });
          }
        }
      }
    });
  };
}